﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Reflection;

/*
Converter class from Linquify by Primary Objects - Business Class Generator
Copyright (C) 2010 Kory Becker
http://www.primaryobjects.com

This class is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace BingMapTest.DTO
{
    /// <summary>
    /// Converter utility class to copy member values from one class to another class with the same member names. Code taken from Linquify by Primary Objects.
    /// </summary>
    public static class Converter
    {
        /// <summary>
        /// Convert originalObject to a businessType object by copying all values from businessType to the same
        /// named fields in originalObject.
        /// </summary>
        /// <typeparam name="TEntity1">Type of originalObject to copy into</typeparam>
        /// <param name="originalObject">Class to copy values into</param>
        /// <param name="abusinessType">Class to copy values from</param>
        /// <returns>originalObject</returns>
        public static object ConvertToBusinessObject<TEntity1>(TEntity1 originalObject, object abusinessType)
        {
            // Convert originalObject to a businessType object.
            object businessObject = null;

            if (originalObject == null || abusinessType == null)
            {
                return businessObject;
            }

            Type businessType = originalObject.GetType();

            // Find the class
            Type obj = Type.GetType(businessType.AssemblyQualifiedName);

            // Get it's constructor
            ConstructorInfo constructor = obj.GetConstructor(new Type[] { });

            // Invoke it's constructor, which returns an instance of businessType.
            businessObject = constructor.Invoke(null);

            PropertyInfo[] propInfo = abusinessType.GetType().GetProperties();
            foreach (PropertyInfo property in propInfo)
            {
                if (!property.PropertyType.IsArray && property.CanWrite)
                {
                    SetPropertyValue(originalObject, property.Name, property.GetValue(abusinessType, null));
                }
            }

            return businessObject;
        }

        private static bool SetPropertyValue(object obj, string linqPropertyName, object propertyValue)
        {
            PropertyInfo[] propInfo = obj.GetType().GetProperties();
            foreach (PropertyInfo property in propInfo)
            {
                string propertyName = property.Name;

                if (!property.PropertyType.IsArray && propertyName.ToLower() == linqPropertyName.ToLower())
                {
                    object currentValue = property.GetValue(obj, null);
                    if (currentValue == propertyValue || (currentValue != null && currentValue.Equals(propertyValue)))
                    {
                        // Value has not changed.
                    }
                    else
                    {
                        try
                        {
                            property.SetValue(obj, propertyValue, null);
                        }
                        catch (Exception excep)
                        {
                        }
                    }

                    return true;
                }
            }

            return false;
        }

        private static object GetPropertyValue(object obj, string linqPropertyName)
        {
            PropertyInfo[] propInfo = obj.GetType().GetProperties();
            foreach (PropertyInfo property in propInfo)
            {
                string propertyName = property.Name;

                if (!property.PropertyType.IsArray && propertyName.ToLower() == linqPropertyName.ToLower())
                {
                    return property.GetValue(obj, null);
                }
            }

            return null;
        }
    }
}
