using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using DB = BingMapTest.DB;

/*
Primary Objects Linquify - Business Class Generator
Copyright (C) 2010 Kory Becker
http://www.primaryobjects.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace BingMapTest.Types.GeneratedTypes.BaseTypes
{
    [Serializable]
    public class location : BusinessBaseType<DB.location, DB.LocationEntities>
    {
        public class Where
        {
			public bool LocationIdSet;
			public bool CreateDateSet;
			public bool DescriptionSet;
			public bool LatitudeSet;
			public bool LongitudeSet;
			public bool TitleSet;

			private System.Nullable<int> _LocationId;
			private global::System.Nullable<global::System.DateTime> _CreateDate;
			private string _Description;
			private System.Nullable<double> _Latitude;
			private System.Nullable<double> _Longitude;
			private string _Title;


			public System.Nullable<int> LocationId { get { return _LocationId; } set { _LocationId = value; LocationIdSet = true; } }
			public global::System.Nullable<global::System.DateTime> CreateDate { get { return _CreateDate; } set { _CreateDate = value; CreateDateSet = true; } }
			public string Description { get { return _Description; } set { _Description = value; DescriptionSet = true; } }
			public System.Nullable<double> Latitude { get { return _Latitude; } set { _Latitude = value; LatitudeSet = true; } }
			public System.Nullable<double> Longitude { get { return _Longitude; } set { _Longitude = value; LongitudeSet = true; } }
			public string Title { get { return _Title; } set { _Title = value; TitleSet = true; } }


            public string MyWhere = "", MyOrderBy = "", MyTop = "";

            public Where()
            {
				LocationId = null;
				CreateDate = null;
				Description = null;
				Latitude = null;
				Longitude = null;
				Title = null;

				LocationIdSet = false;
				CreateDateSet = false;
				DescriptionSet = false;
				LatitudeSet = false;
				LongitudeSet = false;
				TitleSet = false;

            }
        };

		public int LocationId { get; set; }
		public global::System.Nullable<global::System.DateTime> CreateDate { get; set; }
		public string Description { get; set; }
		public double Latitude { get; set; }
		public double Longitude { get; set; }
		public string Title { get; set; }

        public Where WhereClause { get; set; }

        #region Constructors

        public location()
        {
            WhereClause = new Where();
        }

        public location(System.Nullable<int> _LocationId, global::System.Nullable<global::System.DateTime> _CreateDate, string _Description, double _Latitude, double _Longitude, string _Title)
            : this()
        {
            if (_LocationId != null)
            {
                LocationId = (int)_LocationId;
            }

			CreateDate = _CreateDate;
			Description = _Description;
			Latitude = _Latitude;
			Longitude = _Longitude;
			Title = _Title;

        }

        public location(DB.location locationLinq)
            : this()
        {
            location location = new location();
            Converter.ConvertToBusinessObject<location>(location, locationLinq);

            Initialize(location);
        }

        #endregion

        private void Initialize(location location)
        {
			LocationId = location.LocationId;
			CreateDate = location.CreateDate;
			Description = location.Description;
			Latitude = location.Latitude;
			Longitude = location.Longitude;
			Title = location.Title;

            OnInitializeComplete();
        }

        #region Linq Helpers

        public bool Save()
        {
			OnSave();
			
            if (Save<int, location>(this, LocationId, false))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
        }

		public bool Insert()
		{
			// Allows you to force an Insert via the Save function, rather than an Update. Aka, explicit insert.
			// Useful for cases where you want to assign a primary key before inserting, rather than auto-assigning.
			OnSave();
			
            if (Save<int, location>(this, LocationId, true))
            {
				OnSaveComplete();			
				return true;
			}
			
			return false;
		}
		
        public bool Load(int _LocationId)
        {
            DB.location locationLinq = Load<int>(_LocationId);
            
            if (locationLinq == null)
            {
				return false;
            }
            
            location location = new location();
            Converter.ConvertToBusinessObject<location>(location, locationLinq);
            
            Initialize(location);
            
            return true;
        }

        public bool Delete()
        {
	    OnDelete();

            DB.location locationLinq = Delete<int>(LocationId);

			if (locationLinq == null)
			{
				return false;
			}
			
            location location = new location();
            Converter.ConvertToBusinessObject<location>(location, locationLinq);

            Initialize(location);
            
	    OnDeleteComplete();

            return true;
        }

        public List<location> Find()
        {
            List<DB.location> listLinq = Find(WhereClause);
            
            List<location> list = new List<location>();
            foreach (DB.location location in listLinq)
            {
                list.Add(new location(location));
            }

            return list;
        }

        public new List<location> ToList()
        {
            List<DB.location> listLinq = base.ToList();

            List<location> list = new List<location>();
            foreach (DB.location location in listLinq)
            {
                list.Add(new location(location));
            }

            return list;
        }

        #endregion

        #region Overridables

        /// <summary>
        /// Fired after the object is created or loaded.
        /// </summary>
        protected virtual void OnInitializeComplete()
        {
			WhereClause.LocationId = LocationId;
        }

        /// <summary>
        /// Fired before the object is inserted or updated.
        /// </summary>
        protected virtual void OnSave()
        {
        }

        /// <summary>
        /// Fired after the object is inserted or updated.
        /// </summary>
        protected virtual void OnSaveComplete()
        {
			WhereClause.LocationId = LocationId;
        }

        /// <summary>
        /// Fired before the object is deleted.
        /// </summary>
	protected virtual void OnDelete()
	{
	}

        /// <summary>
        /// Fired after the object is deleted.
        /// </summary>
	protected virtual void OnDeleteComplete()
	{
	}

        #endregion
    }
}
