﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using System.Text;
using BingMapTest.DTO;
using BingMapTest.Types.GeneratedTypes;

/*
Example by Kory Becker
http://www.primaryobjects.com

This class is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace BingMapTest.Web
{
    [ServiceContract(Namespace = "")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class DataService
    {
        [OperationContract]
        public List<LocationViewModel> GetLocations()
        {
            List<LocationViewModel> locationList = new List<LocationViewModel>();
            location location = new location();

            // Convert each location to a LocationViewModel DTO.
            foreach (location aLocation in location.ToList())
            {
                locationList.Add(CreateDTOFromLocation(aLocation));
            }

            return locationList;
        }

        [OperationContract]
        public bool DeleteLocation(LocationViewModel locationViewModel)
        {
            location location = CreateLocationFromDTO(locationViewModel);

            if (location.LocationId > 0)
            {
                return location.Delete();
            }
            else
            {
                return false;
            }
        }

        [OperationContract]
        public int SaveLocation(LocationViewModel locationViewModel)
        {
            location location = CreateLocationFromDTO(locationViewModel);
            location.Save();

            return location.LocationId;
        }

        #region DTO Helpers

        private location CreateLocationFromDTO(LocationViewModel locationViewModel)
        {
            // Helper function to convert a LocationViewModel DTO into a location type.
            location location = new location();

            // Copy the location values from the DTO.
            Converter.ConvertToBusinessObject<location>(location, locationViewModel);

            return location;
        }

        private LocationViewModel CreateDTOFromLocation(location location)
        {
            // Helper function to convert a location into a LocationViewModel DTO.
            LocationViewModel locationViewModel = new LocationViewModel(location);

            return locationViewModel;
        }

        #endregion
    }
}
